/*
 *  Controllers.cpp
 *  toycars
 *
 *  Created by Ruben Henner Zilibowitz on 28/06/08.
 *  Copyright 2008 Ruben Henner Zilibowitz. All rights reserved.
 *
 */

#include "Controllers.h"
#include "CarGame.h"
#include "TcPreferences.h"
#include <cstdio>

extern TcPreferences *gPreferences;
extern CarGame *gGameP;

int mapJoystickDeviceToPlayer(int which)
{
   int i;
   
   for (i = 1; i < 4; i++) {
      if (gGameP)
         if (gGameP->isPlayerHuman(i-1) == false)
            continue;
      if (gPreferences->getControllerType(i) >= kXBox360Controller) {
         if (which == 0)
            return i;
         which--;
      }
   }
   
   return 0;
}

JoystickButton mapJoyButtonInput(int btn, ControllerType type)
{
	switch (type) {
      case kXBox360Controller:
         switch (btn) {
         #ifdef WIN32
            case 0: return kAButton;
            case 1: return kBButton;
            case 2: return kXButton;
            case 3: return kYButton;
            case 4: return kLeftShoulderButton;
            case 5: return kRightShoulderButton;
            case 6: return kBackButton;
            case 7: return kStartButton;
         #else
            case 0: return kUpButton;
            case 1: return kDownButton;
            case 2: return kLeftButton;
            case 3: return kRightButton;
            case 4: return kStartButton;
            case 5: return kBackButton;
            case 11: return kAButton;
            case 12: return kBButton;
            case 13: return kXButton;
            case 14: return kYButton;
         #endif
		   }
      case kPlaystation3Controller:
         switch (btn) {
            case 2: return kStartButton;
            case 1: return kBackButton;
            case 14: return kAButton;
		   }
         break;
	}
   return (JoystickButton)(-1);
}

JoystickAxis mapJoyAxisInput(int axis, ControllerType type, short inAxisValue, short *outAxisValue)
{
	//printf("axis: %d value: %d\n", axis, inAxisValue);
   *outAxisValue = inAxisValue;
	switch (type) {
	   case kXBox360Controller:
		   switch (axis) {
         #ifdef WIN32
            case 0: return kLeftHorizontalAxis;
            case 1: return kLeftVerticalAxis;
            case 2:
               if (inAxisValue >= 0) {
                  *outAxisValue = inAxisValue*2 - 32767;
                  //printf("left trigger out value: %d\n", *outAxisValue);
                  return kLeftTrigger;
               }
               else {
                  *outAxisValue = -inAxisValue*2 - 32767;
                  //printf("right trigger out value: %d\n", *outAxisValue);
                  return kRightTrigger;
               }
            case 3: return kRightVerticalAxis;
            case 4: return kRightHorizontalAxis;
         #else
            case 0: return kLeftHorizontalAxis;
            case 1: return kLeftVerticalAxis;
            case 2: return kRightHorizontalAxis;
            case 3: return kRightVerticalAxis;
            case 4: return kLeftTrigger;
            case 5: return kRightTrigger;
         #endif
		   }
      case kPlaystation3Controller:
		   switch (axis) {
            case 2: return kLeftHorizontalAxis;
            case 1: return kLeftVerticalAxis;
            case 0: return kRightHorizontalAxis;
            case 3:
               if(inAxisValue > 0)
               {
                  int value = ((int)inAxisValue)*2 - 32767;
                  if( value > 32766 )
                  {
                     value = 32766;
                  }
                  *outAxisValue = value;
                  printf("right trigger out value: %d\n", *outAxisValue);
                  return kRightTrigger;
               }
               else
               {
                  int value = -(int)inAxisValue*2 - 32767;
                  if( value > 32766 )
                  {
                      value = 32766;
                  }
                  *outAxisValue = value;
                  printf("left trigger out value: %d\n", *outAxisValue);
                  return kLeftTrigger;
               }
		   }
         break;
	}
   return (JoystickAxis)(-1);
}
