/*
	Copyright (C) 2005 by Ruben Henner Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the license.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

#ifndef GAMECONSTANTS_H
#define GAMECONSTANTS_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "Tuple.h"

//
// The scale of the simulation
//

const double kScale = 55;   // pixels / metre

//
// Other constants
//

const double kPi = 3.14159265358979323846;
const double kTwoPi = 2.0 * kPi;
const double kInvTwoPi = 1.0 / kTwoPi;
const double kPiOn2 = 0.5 * kPi;

///--------------------------------------------------------------------------------------
//  sign_smooth
//
//  This utility function smooths the discontinuity in the sign function which
//  is very helpfull when you want to be solving an ODE containing such things.
///--------------------------------------------------------------------------------------

inline double sign_smooth(double x, double k = 1.0)
{
	if (-k < x && x < k)
		return x / k;
	
	if (x > 0.0)
		return 1.0;
	
	// x < 0.0
	return -1.0;
}

///--------------------------------------------------------------------------------------
//  reduce_by_at_most
//
//  This utility function adds or subtracts the value in r from val making it closer to
//  zero but never changing its sign. If |val| < r then it sets val equal to zero.
///--------------------------------------------------------------------------------------

inline void reduce_by_at_most(double& val, double r)
{
	if (val > r)
		val -= r;
	else if (val < -r)
		val += r;
	else
		val = 0.0;
}

const int kWallTile = 25;	// the id of the wall tile
const int kStartRoadTiles = 30;	// the id of the first offroad tile
const int kStartOffroadTiles = 0;	// the id of the first offroad tile
const int kNumTiles = 55;

//#define MOUSE_CONTROL_PLAYER_1	0	// use the mouse to control player 1?
//#define GAMEPAD_CONTROL_PLAYER_1	1	// use an xbox 360 gamepad to control player 1?

#if !(HAVE_FMOD_LIB)
namespace FMOD {
typedef int System;
typedef int Sound;
typedef int Channel;
}
typedef int FMOD_RESULT;
#endif

enum SignType {
   kEasyLeft, kEasyRight, kHardLeft, kHardRight, kEasyLeftThenRight, kEasyRightThenLeft, kHardLeftThenRight, kHardRightThenLeft, kLRLR, kRLRL
};

struct RoadSignInterval {
   SignType type;       // the road sign
   double start, end;   // distance along path from start vertex
   Tuple startPoint, endPoint;
   
   RoadSignInterval() {}
   RoadSignInterval(SignType s, double st, double en, Tuple sp, Tuple ep) : type(s), start(st), end(en), startPoint(sp), endPoint(ep) {}
};

#endif
