/*
	Copyright (C) 2005 by Ruben Henner Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the license.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/
/*
 *  Created by Ruben Henner Zilibowitz on 17/12/05.
 */

#ifndef HOTPOTATOGAME_H
#define HOTPOTATOGAME_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "CarGame.h"

// the minimum rel. norm. velocity required in a collision
// to tag the other player.
const double kMinTagVelocity = 4.0;

class HotPotatoGame : public CarGame
{
public:
            HotPotatoGame(Tileset* inTilemap, Player** inPlayers, int inNumPlayers, FMOD::System* inFMOD_system, Tuple inStartLineA, Tuple inStartLineB, bool inClockwise, double inFuseTime, const list<Tuple> *inPathPolygon, const list<RoadSignInterval> *inSignIntervals);
    virtual ~HotPotatoGame();
    
	void	detonate(Player* player);
	virtual void	renderEveryTexture();
	
	const Player*	getPlayerWithBomb() { return playerWithBomb; }
protected:
	PlayerBomb* bomb;
	Player*		playerWithBomb;
	double		bombFuseLength;
	FMOD::Sound *explosionSound;
    
    virtual void    collisionAction(list<Contact> &contacts, Player* playerI, Player* playerJ);
	virtual void	drawTime(double theTime);
	virtual void drawDisplay(Player *player, const ScRect& viewRect, double dt);
	void drawLose(Player *player);
	void drawWin(Player *player);
};

#endif
