/*
 *  KnockOutGame.cpp
 *  ToyCars
 *
 *  Created by Ruben Henner Zilibowitz on 17/12/05.
 *  Copyright 2005 Ruben Henner Zilibowitz. All rights reserved.
 *
 */

#include "KnockOutGame.h"
#include "GameConstants.h"
#include "ScException.h"
#include "glf.h"
#include "TcPreferences.h"
#include <cstdio>

extern TcPreferences *gPreferences;

KnockOutGame::KnockOutGame(Tileset* inTilemap, Player** inPlayers, int inNumPlayers, FMOD::System* inFMOD_system, Tuple inStartLineA, Tuple inStartLineB, bool inClockwise, const list<Tuple> *inPathPolygon, const list<RoadSignInterval> *inSignIntervals)
    :   CarGame(inTilemap, inPlayers, inNumPlayers, inFMOD_system, inStartLineA, inStartLineB, inClockwise, inPathPolygon, inSignIntervals),
        scores(new int[inNumPlayers])
{
/*
	PlayerEventKey playerEvent;
	playerEvent.type = kPlayerKeyDown;
	playerEvent.key = kThrottleMore;
    for (list<Player*>::iterator i = playersInGame.begin(); i != playersInGame.end(); i++)
	{
        allPlayersList.push_back(*i);
		(*i)->doEvent(&playerEvent);
	}
*/
    for (int i = 0; i < numPlayers; i++)
        scores[i] = 0;
}

KnockOutGame::~KnockOutGame()
{
    delete scores;
}

void KnockOutGame::addView(ScPortRect& portRect, int playerFollow)
{
    CarGame::addView(portRect, playerFollow);
    
    Tuple middle(0.0, 0.0);
    
    for (list<Player*>::iterator i = playersInGame.begin(); i != playersInGame.end(); i++) {
        middle.x += (*i)->car.getLocation_wc().x * kScale;
        middle.y += (*i)->car.getLocation_wc().y * kScale;
    }
    middle.x /= playersInGame.size();
    middle.y /= playersInGame.size();
    
    views.back().view->scrollTo(floor(middle.x), floor(middle.y));
}

const double kWinnerScrollingBias = 0.8;	// must be between 0 and 1
const double kOneMinusScrollBias = 1 - kWinnerScrollingBias;

void KnockOutGame::step(double dt)
{
   ScView* view = views.front().view;
   
   // find which player is in front
   double bestDist = -100;
   Player *winning = NULL;
   for (list<Player*>::iterator i = playersInGame.begin(); i != playersInGame.end(); i++) {
      if ((*i)->getPathDistance() > bestDist) {
         winning = *i;
         bestDist = (*i)->getPathDistance();
      }
   }
   
   // scroll the view to the car location
   Tuple middle(0.0, 0.0);
   for (list<Player*>::iterator i = playersInGame.begin(); i != playersInGame.end(); i++) {
      middle += (*i)->getPathLocation() * kScale;
   }
   middle /= playersInGame.size();
   middle = kOneMinusScrollBias*middle + kWinnerScrollingBias*winning->getPathLocation()*kScale;
   middle.x -= (view->getViewRect().left + view->getViewRect().right)*0.5;
   middle.y -= (view->getViewRect().top + view->getViewRect().bottom)*0.5;
   //middle *= 0.05;
   middle *= 20*dt;
   
   // nb: floor the coordinates to prevent "tearing" between the tiles in the image
   view->scrollBy(floor(middle.x), floor(middle.y));
   
   // now call the standard step function
   CarGame::step(dt);
}

void KnockOutGame::drawViews(double dt)
{
    ScView* view = views.front().view;
    
    // bound the view to the animation bounds
    if (view->getViewRect().left < gameRect.left)
        view->scrollBy(gameRect.left - view->getViewRect().left, 0);
    
    if (view->getViewRect().bottom < gameRect.bottom)
        view->scrollBy(0, gameRect.bottom - view->getViewRect().bottom);
    
    if (view->getViewRect().right > gameRect.right)
        view->scrollBy(gameRect.right - view->getViewRect().right, 0);
    
    if (view->getViewRect().top > gameRect.top)
        view->scrollBy(0, gameRect.top - view->getViewRect().top);
    
    // draw scene
    view->draw();
    
    // draw scores
    glLoadIdentity();
    glOrtho(0, view->getViewRect().right - view->getViewRect().left, 0, view->getViewRect().top - view->getViewRect().bottom, -1, 1);
    static char str15[16];
    glDisable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);
    glColor4f(1.0f,1.0f,1.0f,0.5f);
    glPushMatrix();
    glTranslatef(24, gPreferences->getScreenHeight()-96, 0);
    glScalef(10, 10, 1);
    for (int i = 0; i < numPlayers; i++) {
        sprintf(str15, "%d", scores[i]);
        glfDrawSolidString(str15);
        glTranslatef(0, -4.8, 0);
    }
    glPopMatrix();
    
    // check for knock outs
	
    double curLaps, bestLaps = -2;
    Player *winning = NULL;
    for (list<Player*>::iterator i = playersInGame.begin(); i != playersInGame.end(); i++) {
		curLaps = (*i)->getPathDistance();
        if (curLaps > bestLaps) {
            winning = *i;
            bestLaps = curLaps;
        }
    }
    
	const ScRect* carBounds;
   for (list<Player*>::iterator i = playersInGame.begin(); i != playersInGame.end(); i++) {
      carBounds = &(*i)->car.getBounds();
      if (    carBounds->right < view->getViewRect().left ||
             carBounds->left > view->getViewRect().right ||
             carBounds->bottom > view->getViewRect().top ||
             carBounds->top < view->getViewRect().bottom)
      {
         if ((*i) != winning) {
            Player *removal = *i;
            if (i == playersInGame.begin())
               (i = playersInGame.end())--;
            else
               i--;
            playersInGame.remove(removal);
            allPlayersList.remove(removal);
            allPlayersList.push_front(removal);
            //printf("removed player had path distance of %lf\n", removal->getPathDistance());
            removal->car.setVisible(false);
            removal->car.kill();
            removal->setPausedSounds(true);
            if (playersInGame.size() == 1) {
               scores[winning->getPlayerNumber() - 1]++;
               allPlayersList.remove(winning);
               allPlayersList.push_front(winning);
               winning->car.kill();
               restartFromWinner(winning);
               //printf("winning player had path distance of %lf\n", winning->getPathDistance());
               break;
            }
         }
      }
   }
	
	drawLights();
   drawSign(winning, view->getViewRect(), dt);
}

void KnockOutGame::restartFromWinner(Player *winner)
{
    Tuple placement, loc = winner->getPathLocation(), hOffset, vOffset = winner->getPathVector();
	double theta;
	
	vOffset.Normalise();
	hOffset = vOffset.perp();
	theta = atan2(vOffset.y, vOffset.x);
	//loc = loc * (tilemap.getTileWidth() / kScale);
	short count = 0;
    for (list<Player*>::iterator i = allPlayersList.begin(); i != allPlayersList.end(); i++) {
		if (count % 2 == 1) {
			placement = loc - hOffset * (0.55 * (*i)->car.getHeight() / kScale);
			loc -= vOffset * ((*i)->car.getWidth() / kScale);
		}
		else {
			placement = loc + hOffset * (0.55 * (*i)->car.getHeight() / kScale);
		}
        (*i)->car.setLocation(placement);
		(*i)->car.setOrientation(theta);
        (*i)->car.setVisible(true);
		(*i)->setPausedSounds(false);
		count++;
    }
    
    playersInGame.clear();
	winner->updatePathLocation();
    for (list<Player*>::iterator i = allPlayersList.begin(); i != allPlayersList.end(); i++) {
        playersInGame.push_back(*i);
        (*i)->reset(startLineA, startLineB, winner, clockwise);
    }
    
    Tuple middle(0.0, 0.0);
    
    for (list<Player*>::iterator i = playersInGame.begin(); i != playersInGame.end(); i++) {
        middle.x += (*i)->car.getLocation_wc().x * kScale;
        middle.y += (*i)->car.getLocation_wc().y * kScale;
    }
    middle.x /= playersInGame.size();
    middle.y /= playersInGame.size();
    
    views.back().view->scrollTo(floor(middle.x), floor(middle.y));
}
