/*
 *  Menu.cpp
 *  ToyCars
 *
 *  Created by Ruben Henner Zilibowitz on 24/11/05.
 *  Copyright 2005 Ruben Henner Zilibowitz. All rights reserved.
 *
 */

#include "Menu.h"
#include "GameConstants.h"

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif

#if defined(__APPLE__) && defined(__MACH__)
#include <OpenGL/gl.h>	// Header File For The OpenGL32 Library
#else
#include <GL/gl.h>	// Header File For The OpenGL32 Library	// Header File For The OpenGL32 Library
#endif

#include <cmath>

#include "ScException.h"
#include "glf.h"
#include "TcPreferences.h"
#include "Controllers.h"

extern TcPreferences *gPreferences;

const int kMenuWidth = 300;

int Menu::abs(int x)
{
    if (x < 0)
        return -x;
    
    return x;
}

Menu::Menu(char **inItems, int inNumItems, const char * inTitle, const char * inBackgroundImage)
    : items(inItems), numItems(inNumItems), title(inTitle), backgroundImage(inBackgroundImage), curItem(0)
{
    if (backgroundImage == 0) {
      img = 0;
      return;
    }
    
	// Load the image
	img = IMG_Load(backgroundImage);
	if (img == NULL) {
		fprintf(stderr, "SDL_image error : %s\n", IMG_GetError());
		ScThrowErr("Could not load splash screen image file");
	}
}

Menu::~Menu()
{
	// Free sdl image
	SDL_FreeSurface(img);
}

void Menu::drawBackground()
{
	char string64[64];
   
   if (img == 0) {
      glClear(GL_COLOR_BUFFER_BIT);
      return;
   }
   
	// Draw the image
	glPixelStorei(GL_UNPACK_ALIGNMENT, 4);
	glPixelStorei(GL_UNPACK_ROW_LENGTH, 0);
   glLoadIdentity();
	glOrtho(0, gPreferences->getScreenWidth(), 0, gPreferences->getScreenHeight(), -1, 1);
	glPixelZoom(1, -1);
	glRasterPos2f(gPreferences->getScreenWidth()*0.5 - img->w*0.5, gPreferences->getScreenHeight()*0.5 + img->h*0.5);
	glClear(GL_COLOR_BUFFER_BIT);
	glDrawPixels(img->w, img->h, GL_RGB, GL_UNSIGNED_BYTE, img->pixels);
	
	// Draw version number
	sprintf(string64, "v. %s", VERSION);
	glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
	//glfGetStringBounds(string64, &minx, &miny, &maxx, &maxy);
	glTranslatef(gPreferences->getScreenWidth() - 96, 20, 0);
	glScalef(12, 12, 1);
	glfDrawSolidString(string64);
}

void Menu::drawMenu()
{
   drawBackground();
   glColor4f(0.5f, 1.0f, 1.0f, 1.0f);
   glLoadIdentity();
	glOrtho(0, gPreferences->getScreenWidth(), 0, gPreferences->getScreenHeight(), -1, 1);
   if (title) {
      glPushMatrix();
      glTranslatef(gPreferences->getScreenWidth()*0.5 - kMenuWidth/2, gPreferences->getScreenHeight()*0.5 + numItems*32 + 32, 0);
      glScalef(12, 12, 1);
      glfDrawSolidString(title);
      glPopMatrix();
   }
    
    glTranslatef(gPreferences->getScreenWidth()*0.5 - kMenuWidth/2, gPreferences->getScreenHeight()*0.5 - numItems*32, 0);
    
	//int fontID = glfLoadFont("fonts/cricket1.glf");
	//if (fontID == GLF_ERROR)
	//	ScThrowErr("glfLoadFont returned GLF_ERROR");
	//glfSetCurrentFont(fontID);
   
   glLineWidth(3);
   for (int i = numItems-1; i >= 0; i--)
   {
      if (i == numItems-1)
         glBegin(GL_LINE_LOOP);
      else
         glBegin(GL_LINE_STRIP);
      glVertex2f(0, 0);
      glVertex2f(0, 64);
      glVertex2f(kMenuWidth, 64);
      glVertex2f(kMenuWidth, 0);
      glEnd();
      glPushMatrix();
      glTranslatef(24, 32, 0);
      glScalef(12, 12, 1);
      glfDrawSolidString(items[i]);
      glPopMatrix();
      glTranslatef(0, 64, 0);
   }
   
   // draw selected item
   glLoadIdentity();
	glOrtho(0, gPreferences->getScreenWidth(), 0, gPreferences->getScreenHeight(), -1, 1);
   glTranslatef(gPreferences->getScreenWidth()*0.5 - kMenuWidth/2, gPreferences->getScreenHeight()*0.5 - numItems*32 - (curItem - numItems + 1)*64, 0);
   glLineWidth(6);
   glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
   glBegin(GL_LINE_LOOP);
   glVertex2f(0, 0);
   glVertex2f(0, 64);
   glVertex2f(kMenuWidth, 64);
   glVertex2f(kMenuWidth, 0);
   glEnd();
   
   SDL_GL_SwapBuffers();
   //glfUnloadFont();
}

void Menu::nextItem()
{
   if (curItem < numItems-1) {
      curItem++;
      drawMenu();
   }
}

void Menu::prevItem()
{
   if (curItem > 0) {
      curItem--;
      drawMenu();
   }
}

int Menu::doMenu()
{
   int result;
	SDL_Event event;
   JoystickButton btn;
   JoystickAxis axis;
   int playerNum;
   short axisValue;
   
   drawMenu();
   while (1)
   {
      // get pending events
      while (SDL_PollEvent(&event))
      {
         if (event.type == SDL_QUIT)
            return -1;
         
			if (event.type == SDL_KEYDOWN)
			{
				if (event.key.keysym.sym == SDLK_ESCAPE)
               return -1;
				if (event.key.keysym.sym == SDLK_UP) {
               prevItem();
            }
				if (event.key.keysym.sym == SDLK_DOWN) {
               nextItem();
            }
				if (event.key.keysym.sym == SDLK_RETURN) {
               return curItem;
            }
			}
         
			if (event.type == SDL_MOUSEBUTTONDOWN)
            {
                if (abs(event.button.x - gPreferences->getScreenWidth()/2) < kMenuWidth/2)
                {
                    result = (int)floor((event.button.y - gPreferences->getScreenHeight()*0.5 + numItems*32)/64.0);
                    if (result >= 0 && result < numItems)
                        return result;
                }
            }
         if (event.type == SDL_JOYBUTTONDOWN) {
            playerNum = mapJoystickDeviceToPlayer(event.jaxis.which);
            btn = mapJoyButtonInput(event.jbutton.button, gPreferences->getControllerType(playerNum));
				if (btn == kUpButton) {
               prevItem();
            }
				if (btn == kDownButton) {
               nextItem();
            }
				if (btn == kBackButton) {
               return -1;
            }
				if (btn == kAButton || btn == kStartButton) {
               return curItem;
            }
         }
         if (event.type == SDL_JOYAXISMOTION) {
            const short kThresh = 250;
            playerNum = mapJoystickDeviceToPlayer(event.jaxis.which);
				axis = mapJoyAxisInput(event.jaxis.axis, gPreferences->getControllerType(playerNum), event.jaxis.value, &axisValue);
				if (axis == kLeftVerticalAxis || axis == kRightVerticalAxis)
				{
               if (axisValue > 32767 - kThresh)
                  nextItem();
               else if (axisValue < -32767 + kThresh)
                  prevItem();
            }
         }
        }
    }
}
