/*
	Copyright (C) 2005 by Ruben Henner Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the license.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

/*
 *  Created by Ruben Henner Zilibowitz on 8/04/05.
 */

#include "Minimap.h"
#include "GameConstants.h"
#include "TcPreferences.h"

extern TcPreferences *gPreferences;

Minimap::Minimap(const ScTilemap& tilemap, list<Player*>& inPlayersList)
	: playersList(inPlayersList), minimapImage(NULL)
{
	updateMap(tilemap);
}

Minimap::~Minimap()
{
	if (minimapImage)
		delete minimapImage;
}

void Minimap::draw(double where_x, double where_y, double scale)
{
	// draw a minimap
	
	/*
	const double ms = 1.0;
	const double mx = 100;
	const double my = gPreferences->getScreenHeight() - tilemap.getMapHeight()*scale - 20;
	*/
	
	where_y = gPreferences->getScreenHeight() - imageHeight*scale - where_y;
	
	glColor4ub(255, 255, 255, 255);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glLoadIdentity();
	glOrtho(0, gPreferences->getScreenWidth(), 0, gPreferences->getScreenHeight(), -1, 1);
	glTranslatef(where_x, where_y, 0);
	glScalef(scale, scale, 1);
	minimapImage->draw(0, 0);
	
	// draw players in game into minimap
	
	glDisable(GL_TEXTURE_2D);
	glColor4ub(255, 255, 255, 255);
	glLoadIdentity();
	glOrtho(0, gPreferences->getScreenWidth(), 0, gPreferences->getScreenHeight(), -1, 1);
	glTranslatef(where_x, where_y, 0);
	glScaled(scale*xscale, scale*yscale, 1);
	glBegin(GL_QUADS);
	for (list<Player*>::iterator i = playersList.begin(); i != playersList.end(); i++)
	{
      if ((*i)->isAI()) {
         glColor4ub(255, 255, 255, 255);
      }
      else {
         switch ((*i)->getPlayerNumber()) {
            case 1:
               glColor4ub(255, 0, 0, 255);
               break;
            case 2:
               glColor4ub(0, 255, 0, 255);
               break;
            case 3:
               glColor4ub(0, 0, 255, 255);
               break;
            case 4:
               glColor4ub(127, 0, 127, 255);
               break;
            default:
               glColor4ub(127, 127, 0, 255);
               break;
         }
      }
		glVertex2d((*i)->car.getBounds().left, (*i)->car.getBounds().top);
		glVertex2d((*i)->car.getBounds().left, (*i)->car.getBounds().bottom);
		glVertex2d((*i)->car.getBounds().right, (*i)->car.getBounds().bottom);
		glVertex2d((*i)->car.getBounds().right, (*i)->car.getBounds().top);
	}
	glEnd();
}

void Minimap::updateMap(const ScTilemap& tilemap)
{
	int i;
	
	// delete current image if it exists
	if (minimapImage)
		delete minimapImage;
	
	// setup color table
	
	GLfloat rvals[64];
	GLfloat gvals[64];
	GLfloat bvals[64];
	GLfloat avals[64];
	for (i = 0; i < kStartRoadTiles; i++)
	{
		rvals[i] = 1.0;
		gvals[i] = 1.0;
		bvals[i] = 1.0;
		avals[i] = 0;//0.4;
	}
   avals[kWallTile] = 0;//0.6;
	for (i = kStartRoadTiles; i < 64; i++)
	{
		rvals[i] = 1.0;
		gvals[i] = 1.0;
		bvals[i] = 1.0;
		avals[i] = 0.4;
	}
	glPixelMapfv(GL_PIXEL_MAP_I_TO_R, 64, rvals);
	glPixelMapfv(GL_PIXEL_MAP_I_TO_G, 64, gvals);
	glPixelMapfv(GL_PIXEL_MAP_I_TO_B, 64, bvals);
	glPixelMapfv(GL_PIXEL_MAP_I_TO_A, 64, avals);
	glPixelTransferi(GL_MAP_COLOR, 1);
	
	// load minimap
	ScPixelData pix = {tilemap.getMapWidth(), 1, GL_RGBA, tilemap.getMapWidth(), tilemap.getMapHeight(), GL_COLOR_INDEX, GL_UNSIGNED_BYTE, (const char *)tilemap.getMapData()};
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	minimapImage = new ScImage(0, pix, false);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 4);
	glPixelTransferi(GL_MAP_COLOR, 0);
	
	// set constants
	imageWidth = tilemap.getMapWidth();
	imageHeight = tilemap.getMapHeight();
	xscale = 1.0 / tilemap.getTileWidth();
	yscale = 1.0 / tilemap.getTileHeight();
}
