/*
	Copyright (C) 2005 by Ruben Henner Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the license.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

/*
 *  ScView.cpp
 *  Scoobie
 *
 *  Created by Ruben Henner Zilibowitz on 25/11/04.
 *
 */

#include "ScView.h"

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif

#if defined(__APPLE__) && defined(__MACH__)
#include <OpenGL/gl.h>
//#include <OpenGL/glu.h>
#else

#include <GL/gl.h>	// Header File For The OpenGL32 Library
//#include <GL/glu.h>
#endif

#include <cmath>

//#include "Tuple.h"

//ScRect const * ScView::sActiveViewRect = NULL;
ScView const * ScView::sActiveView = NULL;

//
// constructor
//
ScView::ScView(const ScPortRect& inScreen, const ScRect& inView, ScAnimation &inAnimation)
	: screen(inScreen), view(inView), animation(inAnimation), rotation(0), zoom(1)
{
}

//
// resize - resizes the port rect and view rect
//
void ScView::resize(int width, int height)
{
   view.right += width - screen.width;
   view.top += height - screen.height;
   screen.width = width;
   screen.height = height;
}

//
// draw - sets up viewport and ortho before sub layers draw
//
void ScView::draw()
{
	glLoadIdentity();
	glViewport(screen.left, screen.bottom, screen.width, screen.height);
	//glOrtho(view.left, view.right, view.bottom, view.top, -1.0, 1.0);
	const double w = view.right - view.left;
   const double h = view.top - view.bottom;
   const double centrex = 0.5*(view.left + view.right);
   const double centrey = 0.5*(view.top + view.bottom);
   glOrtho(-w*0.5, w*0.5, -h*0.5, h*0.5, -1.0, 1.0);
   glRotated(rotation, 0, 0, 1);
   glScaled(zoom, zoom, 1);
	glTranslated(-centrex, -centrey, 0);
   
//	sActiveViewRect = &view;
	sActiveView = this;
   animation.draw();
	sActiveView = NULL;
//	sActiveViewRect = NULL;
}

//
// scrollTo - scrolls so the given location is centred without changing the zoom
//
void ScView::scrollTo(ScFloat centre_x, ScFloat centre_y)
{
	ScFloat size_x = view.right - view.left;
	ScFloat size_y = view.top - view.bottom;
	
	view.left = centre_x - size_x*0.5;
	view.bottom = centre_y - size_y*0.5;
	view.right = centre_x + size_x*0.5;
	view.top = centre_y + size_y*0.5;
}

//
// scrollBy - scrolls by the given amount
//
void ScView::scrollBy(ScFloat dx, ScFloat dy)
{
	view.left += dx;
	view.bottom += dy;
	view.right += dx;
	view.top += dy;
}
/*
void ScView::computeViewBoundingRect(ScRect *outRect) const
{
	double w = view.right - view.left;
   double h = view.top - view.bottom;
   double centrex = 0.5*(view.left + view.right);
   double centrey = 0.5*(view.top + view.bottom);
   Tuple A(-w*0.5,-h*0.5),B(w*0.5,-h*0.5),C(w*0.5,h*0.5),D(-w*0.5,h*0.5);
   
   A /= zoom;
   B /= zoom;
   C /= zoom;
   D /= zoom;
   
   A.Rotate(rotation * M_PI / 180);
   B.Rotate(rotation * M_PI / 180);
   C.Rotate(rotation * M_PI / 180);
   D.Rotate(rotation * M_PI / 180);
   
   outRect->left = fmin(fmin(A.x,B.x),fmin(C.x,D.x));
   outRect->right = fmax(fmax(A.x,B.x),fmax(C.x,D.x));
   outRect->bottom = fmin(fmin(A.y,B.y),fmin(C.y,D.y));
   outRect->top = fmax(fmax(A.y,B.y),fmax(C.y,D.y));
   
   outRect->left += centrex;
   outRect->right += centrex;
   outRect->bottom += centrey;
   outRect->top += centrey;
}
*/
void ScView::computeViewBoundingRect(ScRect *outRect) const
{
	double w = view.right - view.left;
   double h = view.top - view.bottom;
   double centrex = 0.5*(view.left + view.right);
   double centrey = 0.5*(view.top + view.bottom);
   double Ax = -w*0.5, Ay = -h*0.5, Bx = w*0.5, By = -h*0.5, Cx = w*0.5, Cy = h*0.5, Dx = -w*0.5, Dy = h*0.5;
   
   Ax /= zoom; Ay /= zoom;
   Bx /= zoom; By /= zoom;
   Cx /= zoom; Cy /= zoom;
   Dx /= zoom; Dy /= zoom;
   
   double s = sin(rotation * M_PI / 180);
   double c = cos(rotation * M_PI / 180);
   double tempX, tempY;
   
   tempX = c*Ax - s*Ay;
   tempY = s*Ax + c*Ay;
   Ax = tempX; Ay = tempY;
   
   tempX = c*Bx - s*By;
   tempY = s*Bx + c*By;
   Bx = tempX; By = tempY;
   
   tempX = c*Cx - s*Cy;
   tempY = s*Cx + c*Cy;
   Cx = tempX; Cy = tempY;
   
   tempX = c*Dx - s*Dy;
   tempY = s*Dx + c*Dy;
   Dx = tempX; Dy = tempY;
   
   outRect->left = fmin(fmin(Ax,Bx),fmin(Cx,Dx));
   outRect->right = fmax(fmax(Ax,Bx),fmax(Cx,Dx));
   outRect->bottom = fmin(fmin(Ay,By),fmin(Cy,Dy));
   outRect->top = fmax(fmax(Ay,By),fmax(Cy,Dy));
   
   outRect->left += centrex;
   outRect->right += centrex;
   outRect->bottom += centrey;
   outRect->top += centrey;
}
