/*
	Copyright (C) 2005 by Ruben Henner Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the license.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

/*
 *  Created by Ruben Henner Zilibowitz on 7/12/07.
 */

#include "StartLight.h"
#include "ScException.h"
#include "SDL.h"
#include "SDL_image.h"

StartLight::StartLight(int k) : lightImage(NULL)
{
	ScPixelData lightImageData;
	SDL_Surface* img;
	char *imgName = NULL;
	
	//fprintf(stderr, "doing %d\n", k);
	switch (k) {
		case 0:
			imgName = strdup("images/lights/yellow1.png");
			break;
		case 1:
			imgName = strdup("images/lights/yellow2.png");
			break;
		case 2:
			imgName = strdup("images/lights/green.png");
			break;
	}
	
	if (imgName == NULL)
		ScThrowErr("No such lights image");
	img = IMG_Load(imgName);
	if (img == NULL)
		ScThrowErr("Could not load light image file");
	
	lightImageData.pitch = img->pitch;
	lightImageData.bytes_pp = img->format->BytesPerPixel;
	lightImageData.internalformat = GL_RGBA;
	lightImageData.width = img->w;
	lightImageData.height = img->h;
	lightImageData.format = GL_RGBA;
	lightImageData.type = GL_UNSIGNED_BYTE;
	lightImageData.pixels = (const char*)img->pixels;
	
	lightImage = new ScImage(0, lightImageData, true);
	
	SDL_FreeSurface(img);
	free(imgName);
	
	//fprintf(stderr,"done %d\n", k);
}

StartLight::~StartLight()
{
	delete lightImage;
}

void StartLight::draw(unsigned char alpha)
{
	glColor4ub(255, 255, 255, alpha);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	lightImage->draw(0,0);
}
