/*
	Copyright (C) 2009 by Ruben Henner Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the license.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

/*
 *  StaticImage.cpp
 *  toycars
 *
 *  Created by Ruben Henner Zilibowitz on 17/02/09.
 *  Copyright 2009 Ruben Henner Zilibowitz. All rights reserved.
 *
 */

#include "StaticImage.h"
#include "ScException.h"
#include "SDL.h"
#include "SDL_image.h"

StaticImage::StaticImage(const char *filename) : image(NULL)
{
	ScPixelData imageData;
	SDL_Surface* img;
	
	img = IMG_Load(filename);
	if (img == NULL)
		ScThrowErr("Could not load image file");
	
	imageData.pitch = img->pitch;
	imageData.bytes_pp = img->format->BytesPerPixel;
	imageData.internalformat = GL_RGBA;
	imageData.width = img->w;
	imageData.height = img->h;
	imageData.format = GL_RGBA;
	imageData.type = GL_UNSIGNED_BYTE;
	imageData.pixels = (const char*)img->pixels;
	
	image = new ScImage(0, imageData, true);
	
	SDL_FreeSurface(img);
   
   w = img->w;
   h = img->h;
}

StaticImage::~StaticImage()
{
	delete image;
}

void StaticImage::draw()
{
	glColor4ub(255, 255, 255, 255);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	image->draw(-w*0.5, -h*0.5);
}
