/*
 *  TcPreferences.cpp
 *  ToyCars
 *
 *  Created by Ruben Henner Zilibowitz on 16/12/05.
 *  Copyright 2005 Ruben Henner Zilibowitz. All rights reserved.
 *
 */

#include "TcPreferences.h"
#include "tinyxml.h"
#include "ScException.h"
#include "Menu.h"
#include <cstdio>
#include <cstring>

//void DrawSplash();

TcPreferences::TcPreferences(const char *inFilename)
	: modifiedScreenPrefs(false)
{
	filename = strdup(inFilename);
	TiXmlDocument *xmlDoc = new TiXmlDocument(filename);
	if (!xmlDoc->LoadFile())
	{
        // create default preferences
        
      defaultPrefs();
      savePreferences(filename);
    }
    else
    {
		try {
			TiXmlHandle docHandle(xmlDoc);
			
			// screen element
			
			TiXmlElement *xScreen = docHandle.FirstChildElement("preferences").FirstChild("screen").Element();
			if (!xScreen)
				ScThrowErr("xml preferences could not find screen element");
			if (xScreen->QueryIntAttribute("width", &data.screenwidth) != TIXML_SUCCESS)
				ScThrowErr("xml preferences screen element could not find width attribute");
			if (xScreen->QueryIntAttribute("height", &data.screenheight) != TIXML_SUCCESS)
				ScThrowErr("xml preferences screen element could not find height attribute");
			
			const char *fullscreen_attr = xScreen->Attribute("fullscreen");
			if (fullscreen_attr == NULL)
				ScThrowErr("xml preferences screen element could not find fullscreen attribute");
			if (strcmp(fullscreen_attr, "yes") == 0)
				data.fullscreen = 1;
			else if (strcmp(fullscreen_attr, "no") == 0)
				data.fullscreen = 0;
			else
				ScThrowErr("xml preferences fullscreen attribute has unknown value");
			
			// graphics elements
			
			TiXmlElement *xGraphics = docHandle.FirstChildElement("preferences").FirstChild("graphics").Element();
			if (!xGraphics)
				ScThrowErr("xml preferences could not find graphics element");
			if (xGraphics->QueryIntAttribute("maxParticles", &data.maxParticles) != TIXML_SUCCESS)
				ScThrowErr("xml preferences graphics element could not find maxParticles attribute");
			
			// bomb elements
			
			TiXmlElement *xBomb = docHandle.FirstChildElement("preferences").FirstChild("bomb").Element();
			if (!xBomb)
				ScThrowErr("xml preferences could not find bomb element");
			if (xBomb->QueryDoubleAttribute("time", &data.fuseLength) != TIXML_SUCCESS)
				ScThrowErr("xml preferences bomb element could not find time attribute");
			
			// music element
			
			TiXmlElement *xMusic = docHandle.FirstChildElement("preferences").FirstChild("music").Element();
			if (!xMusic)
				ScThrowErr("xml preferences could not find music element");
			const char *music_attr = xMusic->Attribute("enabled");
			if (music_attr == NULL)
				ScThrowErr("xml preferences music element could not find enabled attribute");
			if (strcmp(music_attr, "yes") == 0)
				data.music = 1;
			else if (strcmp(music_attr, "no") == 0)
				data.music = 0;
			else
				ScThrowErr("xml preferences music attribute has unknown value");
			
			// sounds element
			
			TiXmlElement *xSounds = docHandle.FirstChildElement("preferences").FirstChild("sounds").Element();
			if (!xSounds)
				ScThrowErr("xml preferences could not find sounds element");
			const char *sounds_attr = xSounds->Attribute("enabled");
			if (sounds_attr == NULL)
				ScThrowErr("xml preferences music element could not find enabled attribute");
			if (strcmp(sounds_attr, "yes") == 0)
				data.sounds = 1;
			else if (strcmp(sounds_attr, "no") == 0)
				data.sounds = 0;
			else
				ScThrowErr("xml preferences sounds attribute has unknown value");
         
         // controllers element
			TiXmlElement *xPlayer1 = docHandle.FirstChildElement("preferences").FirstChildElement("controllers").FirstChild("player1").Element();
			TiXmlElement *xPlayer2 = docHandle.FirstChildElement("preferences").FirstChildElement("controllers").FirstChild("player2").Element();
			TiXmlElement *xPlayer3 = docHandle.FirstChildElement("preferences").FirstChildElement("controllers").FirstChild("player3").Element();
			TiXmlElement *xPlayer4 = docHandle.FirstChildElement("preferences").FirstChildElement("controllers").FirstChild("player4").Element();
         if (!xPlayer1) ScThrowErr("xml player1 element not found");
         if (!xPlayer2) ScThrowErr("xml player2 element not found");
         if (!xPlayer3) ScThrowErr("xml player3 element not found");
         if (!xPlayer4) ScThrowErr("xml player4 element not found");
         if (xPlayer1->QueryIntAttribute("type", (int*)&data.playerControllers[0]) != TIXML_SUCCESS)
				ScThrowErr("xml could not player1 element type attribute");
         if (xPlayer2->QueryIntAttribute("type", (int*)&data.playerControllers[1]) != TIXML_SUCCESS)
				ScThrowErr("xml could not player2 element type attribute");
         if (xPlayer3->QueryIntAttribute("type", (int*)&data.playerControllers[2]) != TIXML_SUCCESS)
				ScThrowErr("xml could not player3 element type attribute");
         if (xPlayer4->QueryIntAttribute("type", (int*)&data.playerControllers[3]) != TIXML_SUCCESS)
				ScThrowErr("xml could not player4 element type attribute");
      }
		catch (ScException e)
		{
			e.printMsg();
			printf("Preferences file is corrupt. Creating new preferences file...\n");
			defaultPrefs();
			savePreferences(filename);
		}
    }
    
    delete xmlDoc;
}

TcPreferences::~TcPreferences()
{
	free(filename);
}

void TcPreferences::defaultPrefs()
{
	data.screenwidth = 1024;
	data.screenheight = 768;
	data.fullscreen = false;
	data.maxParticles = 6000;
	data.fuseLength = 60.0;
	data.music = true;
	data.sounds = true;
   data.playerControllers[0] = kKeyboardController;
   data.playerControllers[1] = kKeyboardController;
   data.playerControllers[2] = kKeyboardController;
   data.playerControllers[3] = kKeyboardController;
}

void TcPreferences::savePreferences(const char *filename)
{
	TiXmlDocument *xmlDoc = new TiXmlDocument(filename);
    TiXmlDeclaration decl("1.0", "UTF-8", "yes");
    TiXmlElement prefs("preferences");
    TiXmlElement screen("screen");
    TiXmlElement graphics("graphics");
    TiXmlElement bomb("bomb");
    TiXmlElement musicElem("music");
    TiXmlElement soundsElem("sounds");
    TiXmlElement controllersElem("controllers");
    TiXmlElement player1Elem("player1");
    TiXmlElement player2Elem("player2");
    TiXmlElement player3Elem("player3");
    TiXmlElement player4Elem("player4");
   
    screen.SetAttribute("width", data.screenwidth);
    screen.SetAttribute("height", data.screenheight);
    screen.SetAttribute("fullscreen", data.fullscreen ? "yes" : "no");
    graphics.SetAttribute("maxParticles", data.maxParticles);
    bomb.SetDoubleAttribute("time", data.fuseLength);
    musicElem.SetAttribute("enabled", data.music ? "yes" : "no");
    soundsElem.SetAttribute("enabled", data.sounds ? "yes" : "no");
    
    player1Elem.SetAttribute("type", data.playerControllers[0]);
    player2Elem.SetAttribute("type", data.playerControllers[1]);
    player3Elem.SetAttribute("type", data.playerControllers[2]);
    player4Elem.SetAttribute("type", data.playerControllers[3]);
    
    controllersElem.InsertEndChild(player1Elem);
    controllersElem.InsertEndChild(player2Elem);
    controllersElem.InsertEndChild(player3Elem);
    controllersElem.InsertEndChild(player4Elem);
    prefs.InsertEndChild(screen);
    prefs.InsertEndChild(graphics);
    prefs.InsertEndChild(bomb);
    prefs.InsertEndChild(musicElem);
    prefs.InsertEndChild(soundsElem);
    prefs.InsertEndChild(controllersElem);
    xmlDoc->InsertEndChild(decl);
    xmlDoc->InsertEndChild(prefs);
    xmlDoc->SaveFile();
    delete xmlDoc;
	
	printf("saved prefs name %s particles %d\n", filename, data.maxParticles);
}

const char *TcPreferences::controllerName(ControllerType n)
{
   switch (n) {
      case 0: return "Keyboard";
      case 1: return "XBox 360 Pad";
      case 2: return "Playstation 3 Pad";
   }
   return 0;
}

void TcPreferences::drawControllersPrefs()
{
   char controllerStr[kNumPlayerControllerOptions][256];
   sprintf(controllerStr[0], "P1: %s", controllerName(data.playerControllers[0]));
   sprintf(controllerStr[1], "P2: %s", controllerName(data.playerControllers[1]));
   sprintf(controllerStr[2], "P3: %s", controllerName(data.playerControllers[2]));
   sprintf(controllerStr[3], "P4: %s", controllerName(data.playerControllers[3]));
	char *items[] = {controllerStr[0], controllerStr[1], controllerStr[2], controllerStr[3], "OK"};
	const int kPrefItems = 5;
	const int kOKItem = kPrefItems-1;
   int menuResult;
	Menu menu(items, kPrefItems, 0, "images/title.png");
   
	do {
      sprintf(controllerStr[0], "P1: %s", controllerName(tempData.playerControllers[0]));
      sprintf(controllerStr[1], "P2: %s", controllerName(tempData.playerControllers[1]));
      sprintf(controllerStr[2], "P3: %s", controllerName(tempData.playerControllers[2]));
      sprintf(controllerStr[3], "P4: %s", controllerName(tempData.playerControllers[3]));
		
      //DrawSplash();
		menuResult = menu.doMenu();
		switch (menuResult)
		{
			case 0:
            tempData.playerControllers[0] = (ControllerType)((tempData.playerControllers[0] + 1) % 3);
				break;
			case 1:
            tempData.playerControllers[1] = (ControllerType)((tempData.playerControllers[1] + 1) % 3);
				break;
			case 2:
            tempData.playerControllers[2] = (ControllerType)((tempData.playerControllers[2] + 1) % 3);
				break;
			case 3:
            tempData.playerControllers[3] = (ControllerType)((tempData.playerControllers[3] + 1) % 3);
				break;
         case -1: // pressing escape should cancel
            menuResult = kOKItem;
			case kOKItem:	// OK
				break;
			default:
				break;
		}
	} while (menuResult < kOKItem);
}

void TcPreferences::drawPrefs()
{
   int menuResult;
	char screenDimsStr[256], fullscreenStr[256], maxParticlesStr[256], fuseLengthStr[256], musicStr[256], soundsStr[256];
	char *items[] = {screenDimsStr, fullscreenStr, maxParticlesStr, fuseLengthStr, musicStr, soundsStr, "Controllers", "Cancel", "Save"};
	const int kPrefItems = 9;
	const int kCancelItem = kPrefItems-2;
	const int kSaveItem = kPrefItems-1;
	Menu menu(items, kPrefItems, 0, "images/title.png");
   
   copyPrefsData(&data, &tempData);
	
	do {
		sprintf(screenDimsStr, "Screen: %d x %d", tempData.screenwidth, tempData.screenheight);
		sprintf(fullscreenStr, "Fullscreen: %s", tempData.fullscreen ? "yes" : "no");
		sprintf(maxParticlesStr, "Max Particles: %d", tempData.maxParticles);
		sprintf(fuseLengthStr, "Fuse Length: %.0lf secs", tempData.fuseLength);
		sprintf(musicStr, "Music: %s", tempData.music ? "yes" : "no");
		sprintf(soundsStr, "Sounds: %s", tempData.sounds ? "yes" : "no");
		
        //DrawSplash();
		menuResult = menu.doMenu();
		switch (menuResult)
		{
			case 0:
				if (tempData.screenwidth == 800) {
					tempData.screenwidth = 1024;
					tempData.screenheight = 768;
				}
				else if (tempData.screenwidth == 1024) {
					tempData.screenwidth = 1280;
					tempData.screenheight = 800;
            }
				else if (tempData.screenwidth == 1280) {
					tempData.screenwidth = 1680;
					tempData.screenheight = 1050;
            }
            else {
					tempData.screenwidth = 800;
					tempData.screenheight = 600;
				}
				break;
			case 1:
				tempData.fullscreen = !tempData.fullscreen;
				break;
			case 2:
				if (tempData.maxParticles < 10000)
					tempData.maxParticles += 1000;
				else
					tempData.maxParticles = 1000;
				break;
			case 3:
				if (tempData.fuseLength < 180)
					tempData.fuseLength += 15;
				else
					tempData.fuseLength = 5;
				break;
				break;
			case 4:
				tempData.music = !tempData.music;
				break;
			case 5:
				tempData.sounds = !tempData.sounds;
				break;
			case 6:
				drawControllersPrefs();
				break;
			
         case -1: // pressing escape should cancel
            menuResult = kCancelItem;
			case kCancelItem:	// Cancel
				break;
			case kSaveItem:	// Save
				modifiedScreenPrefs = tempData.screenwidth != data.screenwidth ||
									tempData.screenheight != data.screenheight ||
									tempData.fullscreen != data.fullscreen;
            copyPrefsData(&tempData, &data);
				savePreferences(filename);
				break;
			default:
				break;
		}
	} while (menuResult < kCancelItem);
}

void TcPreferences::copyPrefsData(PreferencesData *src, PreferencesData *dst)
{
   int i;
   dst->screenwidth = src->screenwidth;
   dst->screenheight = src->screenheight;
   dst->fullscreen = src->fullscreen;
	dst->maxParticles = src->maxParticles;
	dst->fuseLength = src->fuseLength;
	dst->music = src->music;
	dst->sounds = src->sounds;
   for (i = 0; i < kNumPlayerControllerOptions; i++)
      dst->playerControllers[i] = src->playerControllers[i];
}
