/*
	Copyright (C) 2005 by Ruben Henner Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the license.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/
/*
 *  Created by Ruben Henner Zilibowitz on 17/08/06.
 */

#ifndef PATHLINE_H
#define PATHLINE_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "ScPanel.h"
#include "Tuple.h"
#include "GameConstants.h"

const double kVertexRadius = 3;

class PathLine : public ScPanel
{
public:
	PathLine(const ScRect& inBounds, const list<Tuple> *inPathPolygon)
		: ScPanel(inBounds), pathPolygon(inPathPolygon)
	{}
	~PathLine() {}
	
protected:
	const list<Tuple> *pathPolygon;
	
	void draw()
	{
      glLineWidth(6);
		glColor3f(1.0, 0.0, 0.0);
		glDisable(GL_TEXTURE_2D);
		glEnable(GL_TEXTURE_1D);
		glBegin(GL_LINE_LOOP);
		for (list<Tuple>::const_iterator i = pathPolygon->begin(); i != pathPolygon->end(); i++) {
			glVertex2d(i->x * kScale, i->y * kScale);
		}
		glEnd();
      
		glColor3f(1, 1, 1);
		for (list<Tuple>::const_iterator i = pathPolygon->begin(); i != pathPolygon->end(); i++) {
         glBegin(GL_POLYGON);
         for (double a = 0; a < 1; a += 0.1) {
            glVertex2d(i->x * kScale + kVertexRadius*cos(2*M_PI*a),
                       i->y * kScale + kVertexRadius*sin(2*M_PI*a));
         }
         glEnd();
      }
   }
};

#endif
