
#ifndef TRACKVIEW_H
#define TRACKVIEW_H 1
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <FL/Fl.H>
#include <FL/Fl_Gl_Window.H>
#include <FL/gl.h>

#include "ScTilemap.h"
//#include "Minimap.h"

#include <list>
using namespace std;
#include "Tuple.h"
#include "PathLine.h"
#include "Startline.h"
#include "ScView.h"
#include "ScSprite.h"
#include <stdlib.h>

struct SearchDirectories {
   char path[1024];
   SearchDirectories *next;
};

enum ToolType {
   kDrawTool, kPathTool, kStartlineTool, kScrollTool, kRoadSignTool
};

// forwards declaration
class TrackEditorUI;

class TrackView : public Fl_Gl_Window {

public:
   
   void newMap(int w, int h);
   void loadMap(const char *fullFilename);
   void importImage(const char *fullFilename);
   void saveMap(const char *fullFilename);
   void installMap(const char *name);
   void uninstallMap(const char *name);
   
   TrackView(int x,int y,int w,int h,const char *l=0);
   ~TrackView();
   
    /*The widget class draw() override.
     *
     *The draw() function initialize Gl for another round o f drawing
     * then calls specialized functions for drawing each of the
     * entities displayed in the cube view.
     *
     */
   void draw();
   void setTool(ToolType t);
   void setTile(TileType t) { tile = t; }
   void setSign(SignType s) { sign = s; if (signs) signs->setFrame(s); }
   void setUI(TrackEditorUI *inUI) { ui = inUI; }
   
   void resetImportImageDialog(const char *filename);
   
   void setHorizScrollFrac(double x);
   void setVertScrollFrac(double x);
   
   void flipHorizontal();
   void flipVertical();
   
   void resetScroll();
   
   void toggleClockwise() { clockwise = !clockwise; }
   
private:
   
   /*  Draw the cube boundaries
   *
   *Draw the faces of the cube using the boxv[] vertices, using
   * GL_LINE_LOOP for the faces. The color is #defined by TRACKCOLOR.
   */
   void drawTrack();
   void init();
   ScTilemap* loadTileset(const char* tilesetFilename, short border = 0, short seperation = 0, bool blend = false);
   int handle(int event);
   void resize(int x, int y, int w, int h);  // overloaded
   void boundScroll();
   void updateScrollBars();
   double computeCursorPathPosition();
   void drawSignInterval(RoadSignInterval &interval, bool drawSign = true);
   Tuple computePathLocation(const double loc);
   double computeTotalPathLength();
   double computeNearestPathPosition(Tuple pos);
   
   char dataDir[1024];
   char userDataDir[1024];
   int click_x, click_y;
   int drag_x, drag_y;
   TrackEditorUI *ui;
   ToolType tool;
   TileType tile;
   SignType sign;
   ScAnimation animation;
   Startline *startLine;
   ScSprite *signs;
   ScView *view;
	list<Tuple> pathPolygon;
   SearchDirectories *searchPath;
   ScTilemap *tiles;
   bool clockwise;
   bool hoverPathVertexExists;
   list<Tuple>::iterator hoverPathVertex;
   bool selectedPathVertexExists;
   list<Tuple>::iterator selectedPathVertex;
   bool hoverPathIntermediateVertexExists;
   Tuple hoverPathIntermediateVertex;
   list<Tuple>::iterator hoverPathIntermediateNeighbour;
   
   list<RoadSignInterval> signIntervals;
   double signStart, signEnd;
   bool drawingSignInterval;
   
   RoadSignInterval *signIntervalStartMouse;
   RoadSignInterval *signIntervalEndMouse;
//   Minimap *minimap;
};
#endif
