/*
 *  LoadSpriteFromImage.cpp
 *  toycars_vehicle_editor
 *
 *  Created by Ruben Henner Zilibowitz on 24/02/09.
 *  Copyright 2009 Ruben Henner Zilibowitz. All rights reserved.
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <FL/Fl.H>
#include <FL/Fl_Image.H>
#include <FL/Fl_JPEG_Image.H>
#include <FL/Fl_PNG_Image.H>
#include <FL/Fl_BMP_Image.H>
#include <FL/Fl_PNM_Image.H>
#include "LoadSpriteFromImage.h"
#include "ScException.h"
#include <string.h>

//{jpg,png,bmp,pnm,pbm,pgm,ppm}
Fl_Image* loadImage(const char *filename)
{
   char *suffix = strrchr(filename, '.');
   if (suffix == NULL)
      return NULL;
   if (strcmp(suffix,".jpg") == 0)
      return new Fl_JPEG_Image(filename);
   else if (strcmp(suffix,".png") == 0)
      return new Fl_PNG_Image(filename);
   else if (strcmp(suffix,".bmp") == 0)
      return new Fl_BMP_Image(filename);
   else if (strcmp(suffix,".pnm") == 0)
      return new Fl_PNM_Image(filename);
   else if (strcmp(suffix,".pbm") == 0)
      return new Fl_PNM_Image(filename);
   else if (strcmp(suffix,".pgm") == 0)
      return new Fl_PNM_Image(filename);
   else if (strcmp(suffix,".ppm") == 0)
      return new Fl_PNM_Image(filename);/*
   else if (strcmp(suffix,".gif") == 0)
      return new Fl_GIF_Image(filename);
   else if (strcmp(suffix,".xpm") == 0)
      return new Fl_XPM_Image(filename);*/
   else
      return NULL;
}

void PixelDataForImage(const Fl_Image* src, ScPixelData& dst)
{
	dst.bytes_pp = src->d();
	dst.width = src->w();
	dst.height = src->h();
	dst.pitch = dst.width * dst.bytes_pp;
	
	switch (dst.bytes_pp)
	{
		case 4:
			dst.format = GL_RGBA;
			dst.type = GL_UNSIGNED_BYTE;
			dst.internalformat = GL_RGBA;
			break;
		
		case 3:
			dst.format = GL_RGB;
			dst.type = GL_UNSIGNED_BYTE;
			dst.internalformat = GL_RGB;
			break;
		
		default:
			ScThrowErr("Only 32 bit or 24 bit is supported at present");
			break;
	}
	
   if (src->count() == 1)
      dst.pixels = src->data()[0];
   else
      ScThrowErr("Unsupported image type");
}

ScSprite *loadSpriteFromImage(const char *filename, int framewidth, int frameheight, SearchDirectories *dirs)
{
   Fl_Image *img;
   ScPixelData data;
   
   if (strncmp(filename,"$data/",6) == 0) {
      char str[1024];
      bool found = findDataFile(filename+6,dirs,str);
      if (!found)
         ScThrowErr("File not found");
      img = loadImage(str);
   }
   else {
      img = loadImage(filename);
   }
   PixelDataForImage(img, data);
   return new ScSprite(framewidth, frameheight, false, data);
}
