/*
 *  SearchDirectories.cpp
 *  toycars_vehicle_editor
 *
 *  Created by Ruben Henner Zilibowitz on 26/02/09.
 *  Copyright 2009 Ruben Henner Zilibowitz. All rights reserved.
 *
 */

#include "SearchDirectories.h"
#include <cstdio>
#include <cstdlib>
#include <cstring>
#ifdef XCODE
   #include <Carbon/Carbon.h>
#endif

SearchDirectories *gSearchDirs;

void setGlobalSearchDirs()
{
   gSearchDirs = createDataSearchDirs();
}

void freeGlobalSearchDirs()
{
   freeSearchDirectories(gSearchDirs);
}

void freeSearchDirectories(SearchDirectories *dirs)
{
   SearchDirectories *temp;
   while (dirs != NULL) {
      temp = dirs->next;
      free(dirs);
      dirs = temp;
   }
}

SearchDirectories *makeSearchDirectories(const char *path, SearchDirectories *next)
{
   SearchDirectories *result = new SearchDirectories;
   strncpy(result->path, path, 255);
   result->next = next;
   return result;
}

SearchDirectories *createDataSearchDirs()
{
   char dataDir[1024];
   char userDataDir[1024];
   SearchDirectories *searchPath;
#ifdef WIN32
   strncpy(dataDir, DATA_DIR, 1023);
   strncpy(userDataDir, DATA_DIR, 1023);
   searchPath = makeSearchDirectories(dataDir, NULL);
#else
 #ifdef XCODE
   // in the Xcode bundle DATA_DIR is only a relative directory so we need to make it absolute here.
   Boolean succ = CFURLGetFileSystemRepresentation(CFBundleCopyBundleURL(CFBundleGetMainBundle()), TRUE, (UInt8*)dataDir, 1023);
   //getcwd(dataDir, 1023 - strlen(DATA_DIR) - 1);
   if (!succ) {
      fprintf(stderr, "Error: Failed to get application bundle path\n");
      exit(1);
   }
   char *s = strrchr(dataDir, '/');
   s[1] = '\0';
   strcat(dataDir, DATA_DIR);
 #else
   strncpy(dataDir, DATA_DIR, 1023);
 #endif // XCODE
   strcat(dataDir,"/data");
   sprintf(userDataDir, "%s/.toycars", getenv("HOME"));
   searchPath = makeSearchDirectories(dataDir, makeSearchDirectories(userDataDir, NULL));
#endif // WIN32
   return searchPath;
}

bool findDataFile(const char *filename, SearchDirectories *dirs, char *outPath)
{
   const int filenameLen = strlen(filename);
   char str[1024];
   FILE *fp;
   while (dirs != NULL) {
      if (strlen(dirs->path) + filenameLen + 1 < 1023) {
         strcpy(str, dirs->path);
         strcat(str, "/");
         strcat(str, filename);
         fp = fopen(str,"r");
         if (fp) {
            fclose(fp);
            strcpy(outPath,str);
            return true;
         }
      }
      dirs = dirs->next;
   }
   return false;
}
