/*
 *  VehicleObject.h
 *  toycars_vehicle_editor
 *
 *  Created by Ruben Henner Zilibowitz on 23/02/09.
 *  Copyright 2009 Ruben Henner Zilibowitz. All rights reserved.
 *
 */

#ifndef VEHICLEOBJECT_H
#define VEHICLEOBJECT_H 1
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <vector>
#include <list>

struct Vec2D {
   double x, y;
   Vec2D(double xx=0, double yy=0) {x=xx;y=yy;}
};

typedef std::vector< std::list<Vec2D> > Geometry;

class VehicleObject {
public:
      VehicleObject();
      ~VehicleObject();
   
   void reset();
   void loadVehicle(const char *filename);
   void saveVehicle(const char *filename);
   void installVehicle(const char *name);
   void uninstallVehicle(const char *name);
   
   void loadSpriteFromDataDir(const char *name);
   void loadWheelSpriteFromDataDir(const char *name);
   
   double getAxelWidth() { return axelWidth; }
   double getFrontAxel() { return frontAxel; }
   double getRearAxel() { return rearAxel; }
   double getFrontWheelInset() { return frontWheelInset; }
   double getFrontWheelRadius() { return frontWheelRadius; }
   double getRearWheelRadius() { return rearWheelRadius; }
   double getMass() { return mass; }
   double getInertia() { return inertia; }
   double getFrontWheelInertia() { return frontWheelInertia; }
   double getRearWheelInertia() { return rearWheelInertia; }
   double getDrag() { return drag; }
   double getRollingResistance() { return rollingResistance; }
   int getVehicleFrameWidth() { return vehicleFrameWidth; }
   int getVehicleFrameHeight() { return vehicleFrameHeight; }
   int getWheelFrameWidth() { return wheelFrameWidth; }
   int getWheelFrameHeight() { return wheelFrameHeight; }
   bool getWheelsAboveSprite() { return wheelsAboveSprite; }
   Geometry *getGeometry() { return &geometry; }
   double *getFrontSlipRatioPacejka() { return frontSlipRatioPacejka; }
   double *getFrontSlipAnglePacejka() { return frontSlipAnglePacejka; }
   double *getRearSlipRatioPacejka() { return rearSlipRatioPacejka; }
   double *getRearSlipAnglePacejka() { return rearSlipAnglePacejka; }
   char *getVehicleSpriteFilename() { return vehicleSpriteFilename; }
   char *getWheelSpriteFilename() { return wheelSpriteFilename; }
   const char *getTitle() { return title; }
   const char *getAuthor() { return author; }
   
   void setWheelAboveSprite(bool x) { wheelsAboveSprite = x; }
   void setAxelWidth(double x) { axelWidth = x; }
   void setFrontAxel(double x) { frontAxel = x; }
   void setRearAxel(double x) { rearAxel = x; }
   void setFrontWheelInset(double x) { frontWheelInset = x; }
   void setFrontWheelRadius(double x) { frontWheelRadius = x; }
   void setRearWheelRadius(double x) { rearWheelRadius = x; }
   void setMass(double x) { mass = x; }
   void setInertia(double x) { inertia = x; }
   void setFrontWheelInertia(double x) { frontWheelInertia = x; }
   void setRearWheelInertia(double x) { rearWheelInertia = x; }
   void setDrag(double x) { drag = x; }
   void setRollingResistance(double x) { rollingResistance = x; }
   void setVehicleFrameWidth(int x) { vehicleFrameWidth = x; }
   void setVehicleFrameHeight(int x) { vehicleFrameHeight = x; }
   void setWheelFrameWidth(int x) { wheelFrameWidth = x; }
   void setWheelFrameHeight(int x) { wheelFrameHeight = x; }
   void setTitle(const char *str);
   void setAuthor(const char *str);
   
protected:
   char title[256];
   char author[256];
   int vehicleFrameWidth, vehicleFrameHeight;
   char vehicleSpriteFilename[256];
   Geometry geometry;
   double axelWidth;
   double frontAxel;
   double rearAxel;
   double frontWheelInset;
   double frontWheelRadius;
   double rearWheelRadius;
   double mass;
   double inertia;
   double frontWheelInertia;
   double rearWheelInertia;
   double drag;
   double rollingResistance;
   int wheelFrameWidth, wheelFrameHeight;
   bool wheelsAboveSprite;
   char wheelSpriteFilename[256];
   double frontSlipRatioPacejka[4];
   double frontSlipAnglePacejka[4];
   double rearSlipRatioPacejka[4];
   double rearSlipAnglePacejka[4];
};

#endif
